// DynamicTripleTests.cpp
//
// ICS 46 Spring 2022
// Code Example
//
// A handful of Google Test unit tests that demonstrate a few of the
// things that DynamicTriples do.

#include <stdexcept>
#include <string>
#include <gtest/gtest.h>
#include "DynamicTriple.hpp"


TEST(DynamicTripleTests, defaultConstructedTriplesHaveDefaultConstructedValues)
{
    DynamicTriple<std::string, std::string, std::string> t;

    EXPECT_EQ(std::string{""}, t.a());
    EXPECT_EQ(std::string{""}, t.b());
    EXPECT_EQ(std::string{""}, t.c());
}


TEST(DynamicTripleTests, triplesCanBeConstructedWithCopiesOfValues)
{
    DynamicTriple<int, double, unsigned int> t{3, -3.5, 40};

    EXPECT_EQ(3, t.a());
    EXPECT_EQ(-3.5, t.b());
    EXPECT_EQ(40, t.c());
}


TEST(DynamicTripleTests, triplesCanBeClearedBackToDefaultConstructedValues)
{
    DynamicTriple<std::string, std::string, std::string> t{"Boo", "is", "happy"};

    t.clear();

    EXPECT_EQ(std::string{""}, t.a());
    EXPECT_EQ(std::string{""}, t.b());
    EXPECT_EQ(std::string{""}, t.c());
}


namespace
{
    class ThrowingCopyConstructor
    {
    public:
        ThrowingCopyConstructor() { }

        ThrowingCopyConstructor(const ThrowingCopyConstructor& tcc)
        {
            throw std::runtime_error{"Doh"};
        }
    };


    void makeFailingTriple()
    {
        DynamicTriple<ThrowingCopyConstructor, int, int> t{
            ThrowingCopyConstructor{}, 3, 3};
    }
}


TEST(DynamicTripleTests, constructingTriplesFailsWhenCopyingParametersFails)
{
    ASSERT_THROW(makeFailingTriple(), std::runtime_error);
}

